@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -492,7 +492,7 @@ static void print_header(const setup::header & header) {
 	                                                 setup::header::NoPrivileges);
 	std::cout << "Show language dialog: " << color::cyan << header.show_language_dialog
 	          << color::reset << '\n';
-	std::cout << if_not_equal("Danguage detection", header.language_detection,
+	std::cout << if_not_equal("Language detection", header.language_detection,
 	                          setup::header::NoLanguageDetection);
 	std::cout << "Compression: " << color::cyan << header.compression
 	          << color::reset << '\n';
