@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -127,10 +127,10 @@ class file_output : private boost::noncopyable {
 	
 	fs::path path_;
 	const processed_file * file_;
-	util::ofstream stream_;
+	util::fstream stream_;
 	
 	crypto::hasher checksum_;
-	bool checksum_valid_;
+	boost::uint64_t checksum_position_;
 	
 	boost::uint64_t position_;
 	boost::uint64_t total_written_;
@@ -143,14 +143,18 @@ public:
 		: path_(dir / f->path())
 		, file_(f)
 		, checksum_(f->entry().checksum.type)
-		, checksum_valid_(f->entry().checksum.type != crypto::None)
+		, checksum_position_(f->entry().checksum.type == crypto::None ? boost::uint64_t(-1) : 0)
 		, position_(0)
 		, total_written_(0)
 		, write_(write)
 	{
 		if(write_) {
 			try {
-				stream_.open(path_, std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
+				std::ios_base::openmode flags = std::ios_base::out | std::ios_base::binary | std::ios_base::trunc;
+				if(file_->is_multipart()) {
+					flags |= std::ios_base::in;
+				}
+				stream_.open(path_, flags);
 				if(!stream_.is_open()) {
 					throw std::exception();
 				}
@@ -166,8 +170,9 @@ public:
 			stream_.write(data, std::streamsize(n));
 		}
 		
-		if(checksum_valid_) {
+		if(checksum_position_ == position_) {
 			checksum_.update(data, n);
+			checksum_position_ += n;
 		}
 		
 		position_ += n;
@@ -182,33 +187,23 @@ public:
 			return;
 		}
 		
-		checksum_valid_ = false;
-		
 		debug("seeking output from " << print_hex(position_) << " to " << print_hex(new_position));
 		
 		if(!write_) {
+			position_ = new_position;
 			return;
 		}
 		
-		const boost::uint64_t max = boost::uint64_t(std::numeric_limits<util::ofstream::off_type>::max() / 4);
+		const boost::uint64_t max = boost::uint64_t(std::numeric_limits<util::fstream::off_type>::max() / 4);
 		
 		if(new_position <= max) {
-			stream_.seekp(util::ofstream::off_type(new_position), std::ios_base::beg);
+			stream_.seekp(util::fstream::off_type(new_position), std::ios_base::beg);
 		} else {
-			if(new_position > position_) {
-				boost::uint64_t diff = new_position - position_;
-				while(diff > max) {
-					stream_.seekp(util::ofstream::off_type(max), std::ios_base::cur);
-					diff -= max;
-				}
-				stream_.seekp(util::ofstream::off_type(diff), std::ios_base::cur);
-			} else {
-				boost::uint64_t diff = position_ - new_position;
-				while(diff > max) {
-					stream_.seekp(-util::ofstream::off_type(max), std::ios_base::cur);
-					diff -= max;
-				}
-				stream_.seekp(-util::ofstream::off_type(diff), std::ios_base::cur);
+			util::fstream::off_type sign = (new_position > position_) ? 1 : -1;
+			boost::uint64_t diff = (new_position > position_) ? new_position - position_ : position_ - new_position;
+			while(diff > 0) {
+				stream_.seekp(sign * util::fstream::off_type(std::min(diff, max)), std::ios_base::cur);
+				diff -= std::min(diff, max);
 			}
 		}
 		
@@ -232,7 +227,44 @@ public:
 	}
 	
 	bool has_checksum() const {
-		return checksum_valid_ && position_ == file_->entry().size;
+		return checksum_position_ == file_->entry().size;
+	}
+	
+	bool calculate_checksum() {
+		
+		if(has_checksum()) {
+			return true;
+		}
+		
+		if(!write_) {
+			return false;
+		}
+		
+		debug("calculating output checksum for " << path_);
+		
+		const boost::uint64_t max = boost::uint64_t(std::numeric_limits<util::fstream::off_type>::max() / 4);
+		
+		boost::uint64_t diff = checksum_position_;
+		stream_.seekg(util::fstream::off_type(std::min(diff, max)), std::ios_base::beg);
+		diff -= std::min(diff, max);
+		while(diff > 0) {
+			stream_.seekg(util::fstream::off_type(std::min(diff, max)), std::ios_base::cur);
+			diff -= std::min(diff, max);
+		}
+		
+		while(!stream_.eof()) {
+			char buffer[8192];
+			std::streamsize n = stream_.read(buffer, sizeof(buffer)).gcount();
+			checksum_.update(buffer, size_t(n));
+			checksum_position_ += boost::uint64_t(n);
+		}
+		
+		if(!has_checksum()) {
+			log_warning << "Could not read back " << path_ << " to calculate output checksum for multi-part file";
+			return false;
+		}
+		
+		return true;
 	}
 	
 	crypto::checksum checksum() {
@@ -482,12 +514,14 @@ bool insert_dirs(DirectoriesMap & processed_directories, const path_filter & inc
 }
 
 bool rename_collision(const extract_options & o, FilesMap & processed_files, const std::string & path,
-                      const processed_file & other, bool common_component, bool common_language, bool first) {
+                      const processed_file & other, bool common_component, bool common_language,
+                      bool common_arch, bool first) {
 	
 	const setup::file_entry & file = other.entry();
 	
 	bool require_number_suffix = !first || (o.collisions == RenameAllCollisions);
 	std::ostringstream oss;
+	const setup::file_entry::flags arch_flags = setup::file_entry::Bits32 | setup::file_entry::Bits64;
 	
 	if(!common_component && !file.components.empty()) {
 		if(setup::is_simple_expression(file.components)) {
@@ -503,6 +537,13 @@ bool rename_collision(const extract_options & o, FilesMap & processed_files, con
 			}
 		}
 	}
+	if(!common_arch && (file.options & arch_flags) == setup::file_entry::Bits32) {
+		require_number_suffix = false;
+		oss << "@32bit";
+	} else if(!common_arch && (file.options & arch_flags) == setup::file_entry::Bits64) {
+		require_number_suffix = false;
+		oss << "@64bit";
+	}
 	
 	size_t i = 0;
 	std::string suffix = oss.str();
@@ -536,23 +577,26 @@ void rename_collisions(const extract_options & o, FilesMap & processed_files,
 		
 		const processed_file & base = processed_files.find(path)->second;
 		const setup::file_entry & file = base.entry();
+		const setup::file_entry::flags arch_flags = setup::file_entry::Bits32 | setup::file_entry::Bits64;
 		
 		bool common_component = true;
 		bool common_language = true;
+		bool common_arch = true;
 		BOOST_FOREACH(const processed_file & other, collision.second) {
 			common_component = common_component && other.entry().components == file.components;
 			common_language = common_language && other.entry().languages == file.languages;
+			common_arch = common_arch && (other.entry().options & arch_flags) == (file.options & arch_flags);
 		}
 		
 		bool ignore_component = common_component || o.collisions != RenameAllCollisions;
 		if(rename_collision(o, processed_files, path, base,
-		                    ignore_component, common_language, true)) {
+		                    ignore_component, common_language, common_arch, true)) {
 			processed_files.erase(path);
 		}
 		
 		BOOST_FOREACH(const processed_file & other, collision.second) {
 			rename_collision(o, processed_files, path, other,
-			                 common_component, common_language, false);
+			                 common_component, common_language, common_arch, false);
 		}
 		
 	}
@@ -648,10 +692,10 @@ bool print_file_info(const extract_options & o, const setup::info & info) {
 				std::cout << '\n';
 			}
 			if(o.silent) {
-				std::cout << util::encoding_name(info.version.codepage()) << '\n';
+				std::cout << util::encoding_name(info.codepage) << '\n';
 			} else {
 				std::cout << "Password encoding: " << color::yellow
-				          << util::encoding_name(info.version.codepage()) << color::reset << '\n';
+				          << util::encoding_name(info.codepage) << color::reset << '\n';
 			}
 		} else if(!o.quiet) {
 			std::cout << "Setup is not passworded!\n";
@@ -834,27 +878,27 @@ processed_entries filter_entries(const extract_options & o, const setup::info &
 
 } // anonymous namespace
 
-void process_file(const fs::path & file, const extract_options & o) {
+void process_file(const fs::path & installer, const extract_options & o) {
 	
 	bool is_directory;
 	try {
-		is_directory = fs::is_directory(file);
+		is_directory = fs::is_directory(installer);
 	} catch(...) {
-		throw std::runtime_error("Could not open file \"" + file.string()
+		throw std::runtime_error("Could not open file \"" + installer.string()
 		                         + "\": access denied");
 	}
 	if(is_directory) {
-		throw std::runtime_error("Input file \"" + file.string() + "\" is a directory!");
+		throw std::runtime_error("Input file \"" + installer.string() + "\" is a directory!");
 	}
 	
 	util::ifstream ifs;
 	try {
-		ifs.open(file, std::ios_base::in | std::ios_base::binary);
+		ifs.open(installer, std::ios_base::in | std::ios_base::binary);
 		if(!ifs.is_open()) {
 			throw std::exception();
 		}
 	} catch(...) {
-		throw std::runtime_error("Could not open file \"" + file.string() + '"');
+		throw std::runtime_error("Could not open file \"" + installer.string() + '"');
 	}
 	
 	loader::offsets offsets;
@@ -903,8 +947,17 @@ void process_file(const fs::path & file, const extract_options & o) {
 	ifs.seekg(offsets.header_offset);
 	setup::info info;
 	try {
-		info.load(ifs, entries);
-	} catch(const std::ios_base::failure & e) {
+		info.load(ifs, entries, o.codepage);
+	} catch(const setup::version_error &) {
+		fs::path headerfile = installer;
+		headerfile.replace_extension(".0");
+		if(offsets.header_offset == 0 && headerfile != installer && fs::exists(headerfile)) {
+			log_info << "Opening \"" << color::cyan << headerfile.string() << color::reset << '"';
+			process_file(headerfile, o);
+			return;
+		}
+		throw;
+	} catch(const std::exception & e) {
 		std::ostringstream oss;
 		oss << "Stream error while parsing setup headers!\n";
 		oss << " ├─ detected setup version: " << info.version << '\n';
@@ -924,7 +977,7 @@ void process_file(const fs::path & file, const extract_options & o) {
 			log_warning << "Setup contains encrypted files, use the --password option to extract them";
 		}
 	} else {
-		util::from_utf8(o.password, password, info.version.codepage());
+		util::from_utf8(o.password, password, info.codepage);
 		if(info.header.options & setup::header::Password) {
 			crypto::hasher checksum(info.header.password.type);
 			checksum.update(info.header.password_salt.c_str(), info.header.password_salt.length());
@@ -1003,9 +1056,22 @@ void process_file(const fs::path & file, const extract_options & o) {
 		files_for_location[file.entry().location].push_back(output_location(&file, 0));
 		if(o.test || o.extract) {
 			boost::uint64_t offset = info.data_entries[file.entry().location].uncompressed_size;
+			boost::uint32_t sort_slice = info.data_entries[file.entry().location].chunk.first_slice;
+			boost::uint32_t sort_offset = info.data_entries[file.entry().location].chunk.sort_offset;
 			BOOST_FOREACH(boost::uint32_t location, file.entry().additional_locations) {
+				setup::data_entry & data = info.data_entries[location];
 				files_for_location[location].push_back(output_location(&file, offset));
-				offset += info.data_entries[location].uncompressed_size;
+				offset += data.uncompressed_size;
+				if(data.chunk.first_slice > sort_slice ||
+				   (data.chunk.first_slice == sort_slice && data.chunk.sort_offset > sort_offset)) {
+					sort_slice = data.chunk.first_slice;
+					sort_offset = data.chunk.sort_offset;
+				} else if(data.chunk.first_slice == sort_slice && data.chunk.sort_offset == data.chunk.offset) {
+					data.chunk.sort_offset = ++sort_offset;
+				} else {
+					// Could not reorder chunk - no point in trying to reordder the remaining chunks
+					sort_slice = boost::uint32_t(-1);
+				}
 			}
 		}
 	}
@@ -1016,15 +1082,11 @@ void process_file(const fs::path & file, const extract_options & o) {
 	typedef std::map<stream::chunk, Files> Chunks;
 	Chunks chunks;
 	for(size_t i = 0; i < info.data_entries.size(); i++) {
-		if(files_for_location[i].empty()) {
-			continue;
+		if(!files_for_location[i].empty()) {
+			setup::data_entry & location = info.data_entries[i];
+			chunks[location.chunk][location.file] = i;
+			total_size += location.uncompressed_size;
 		}
-		setup::data_entry & location = info.data_entries[i];
-		if(location.chunk.compression == stream::UnknownCompression) {
-			location.chunk.compression = info.header.compression;
-		}
-		chunks[location.chunk][location.file] = i;
-		total_size += location.uncompressed_size;
 	}
 	
 	boost::scoped_ptr<stream::slice_reader> slice_reader;
@@ -1032,8 +1094,8 @@ void process_file(const fs::path & file, const extract_options & o) {
 		if(offsets.data_offset) {
 			slice_reader.reset(new stream::slice_reader(&ifs, offsets.data_offset));
 		} else {
-			fs::path dir = file.parent_path();
-			std::string basename = util::as_string(file.stem());
+			fs::path dir = installer.parent_path();
+			std::string basename = util::as_string(installer.stem());
 			std::string basename2 = info.header.base_filename;
 			// Prevent access to unexpected files
 			std::replace(basename2.begin(), basename2.end(), '/', '_');
@@ -1179,8 +1241,8 @@ void process_file(const fs::path & file, const extract_options & o) {
 			// Open output files
 			boost::ptr_vector<file_output> single_outputs;
 			std::vector<file_output *> outputs;
-			BOOST_FOREACH(const output_location & location, output_locations) {
-				const processed_file * fileinfo = location.first;
+			BOOST_FOREACH(const output_location & output_loc, output_locations) {
+				const processed_file * fileinfo = output_loc.first;
 				try {
 					
 					if(!o.extract && fileinfo->entry().checksum.type == crypto::None) {
@@ -1207,7 +1269,7 @@ void process_file(const fs::path & file, const extract_options & o) {
 					
 					outputs.push_back(output);
 					
-					output->seek(location.second);
+					output->seek(output_loc.second);
 					
 				} catch(boost::bad_pointer &) {
 					// should never happen
@@ -1251,20 +1313,15 @@ void process_file(const fs::path & file, const extract_options & o) {
 				}
 				
 				// Verify output checksum if available
-				if(output->file()->entry().checksum.type != crypto::None) {
-					if(output->has_checksum()) {
-						crypto::checksum checksum = output->checksum();
-						if(checksum != output->file()->entry().checksum) {
-							log_warning << "Output checksum mismatch for " << output->file()->path() << ":\n"
-							            << " ├─ actual:   " << checksum << '\n'
-							            << " └─ expected: " << output->file()->entry().checksum;
-							if(o.test) {
-								throw std::runtime_error("Integrity test failed!");
-							}
+				if(output->file()->entry().checksum.type != crypto::None && output->calculate_checksum()) {
+					crypto::checksum output_checksum = output->checksum();
+					if(output_checksum != output->file()->entry().checksum) {
+						log_warning << "Output checksum mismatch for " << output->file()->path() << ":\n"
+						            << " ├─ actual:   " << output_checksum << '\n'
+						            << " └─ expected: " << output->file()->entry().checksum;
+						if(o.test) {
+							throw std::runtime_error("Integrity test failed!");
 						}
-					} else {
-						// This should not happen
-						log_warning << "Could not verify output checksum of file " << output->file()->path();
 					}
 				}
 				
@@ -1310,7 +1367,7 @@ void process_file(const fs::path & file, const extract_options & o) {
 	}
 	
 	if(o.warn_unused || o.gog) {
-		gog::probe_bin_files(o, info, file, offsets.data_offset == 0);
+		gog::probe_bin_files(o, info, installer, offsets.data_offset == 0);
 	}
 	
 }
