@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -145,6 +145,7 @@ int main(int argc, char * argv[]) {
 	
 	po::options_description modifiers("Modifiers");
 	modifiers.add_options()
+		("codepage", po::value<boost::uint32_t>(), "Encoding for ANSI strings")
 		("collisions", po::value<std::string>(), "How to handle duplicate files")
 		("default-language", po::value<std::string>(), "Default language for renaming")
 		("dump", "Dump contents without converting filenames")
@@ -289,13 +290,13 @@ int main(int argc, char * argv[]) {
 		o.preserve_file_times = true, o.local_timestamps = false;
 		po::variables_map::const_iterator i = options.find("timestamps");
 		if(i != options.end()) {
-			std::string timezone = i->second.as<std::string>();
-			if(boost::iequals(timezone, "none")) {
+			std::string timezone_name = i->second.as<std::string>();
+			if(boost::iequals(timezone_name, "none")) {
 				o.preserve_file_times = false;
-			} else if(!boost::iequals(timezone, "UTC")) {
+			} else if(!boost::iequals(timezone_name, "UTC")) {
 				o.local_timestamps = true;
-				if(!boost::iequals(timezone, "local")) {
-					util::set_local_timezone(timezone);
+				if(!boost::iequals(timezone_name, "local")) {
+					util::set_local_timezone(timezone_name);
 				}
 			}
 		}
@@ -309,6 +310,10 @@ int main(int argc, char * argv[]) {
 		}
 	}
 	
+	{
+		po::variables_map::const_iterator i = options.find("codepage");
+		o.codepage = (i != options.end()) ? i->second.as<boost::uint32_t>() : 0;
+	}
 	{
 		o.collisions = OverwriteCollisions;
 		po::variables_map::const_iterator i = options.find("collisions");
@@ -370,7 +375,7 @@ int main(int argc, char * argv[]) {
 			 */
 			o.output_dir = i->second.as<std::string>();
 			try {
-				if(!o.output_dir.empty() && !fs::exists(o.output_dir)) {
+				if(o.extract && !o.output_dir.empty() && !fs::exists(o.output_dir)) {
 					fs::create_directory(o.output_dir);
 				}
 			} catch(...) {
