@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -39,7 +39,7 @@
 
 namespace setup {
 
-struct version;
+struct info;
 
 struct file_entry : public item {
 	
@@ -113,7 +113,7 @@ struct file_entry : public item {
 	crypto::checksum checksum;
 	boost::uint64_t size;
 	
-	void load(std::istream & is, const version & version);
+	void load(std::istream & is, const info & i);
 	
 };
 
