@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2018 Daniel Scharrer
+ * Copyright (C) 2012-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -79,7 +79,9 @@ std::string filename_map::expand_variables(it & begin, it end, bool close) const
 		while(pos != end && *pos != '{' && *pos != '}') {
 			++pos;
 		}
+		ptrdiff_t obegin = ptrdiff_t(result.size());
 		result.append(begin, pos);
+		result.erase(std::remove_if(result.begin() + obegin, result.end(), is_unsafe_path_char()), result.end());
 		begin = pos;
 		
 		if(pos == end) {
@@ -114,7 +116,7 @@ std::string filename_map::expand_variables(it & begin, it end, bool close) const
 	return result;
 }
 
-std::string filename_map::shorten_path(const std::string & path) const {
+std::string filename_map::shorten_path(const std::string & path) {
 	
 	std::string result;
 	result.reserve(path.size());
