@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -26,7 +26,6 @@
 #include <boost/static_assert.hpp>
 
 #include "setup/version.hpp"
-#include "util/encoding.hpp"
 #include "util/load.hpp"
 #include "util/storedenum.hpp"
 
@@ -136,106 +135,132 @@ STORED_ENUM_MAP(stored_compression_method_3, stream::UnknownCompression,
 	stream::LZMA2
 );
 
+// 6.0.0+
+STORED_FLAGS_MAP(stored_privileges_required_overrides,
+	header::Commandline,
+	header::Dialog
+);
+
 } // anonymous namespace
 
 void header::load(std::istream & is, const version & version) {
 	
 	options = 0;
 	
-	if(version < INNO_VERSION(1, 3, 21)) {
+	if(version < INNO_VERSION(1, 3, 0)) {
 		(void)util::load<boost::uint32_t>(is); // uncompressed size of the setup header
 	}
 	
-	is >> util::encoded_string(app_name, version.codepage());
-	is >> util::encoded_string(app_versioned_name, version.codepage());
-	if(version >= INNO_VERSION(1, 3, 21)) {
-		is >> util::encoded_string(app_id, version.codepage());
+	is >> util::binary_string(app_name);
+	is >> util::binary_string(app_versioned_name);
+	if(version >= INNO_VERSION(1, 3, 0)) {
+		is >> util::binary_string(app_id);
+	} else {
+		app_id.clear();
 	}
-	is >> util::encoded_string(app_copyright, version.codepage());
-	if(version >= INNO_VERSION(1, 3, 21)) {
-		is >> util::encoded_string(app_publisher, version.codepage());
-		is >> util::encoded_string(app_publisher_url, version.codepage());
+	is >> util::binary_string(app_copyright);
+	if(version >= INNO_VERSION(1, 3, 0)) {
+		is >> util::binary_string(app_publisher);
+		is >> util::binary_string(app_publisher_url);
 	} else {
 		app_publisher.clear(), app_publisher_url.clear();
 	}
 	if(version >= INNO_VERSION(5, 1, 13)) {
-		is >> util::encoded_string(app_support_phone, version.codepage());
+		is >> util::binary_string(app_support_phone);
 	} else {
 		app_support_phone.clear();
 	}
-	if(version >= INNO_VERSION(1, 3, 21)) {
-		is >> util::encoded_string(app_support_url, version.codepage());
-		is >> util::encoded_string(app_updates_url, version.codepage());
-		is >> util::encoded_string(app_version, version.codepage());
+	if(version >= INNO_VERSION(1, 3, 0)) {
+		is >> util::binary_string(app_support_url);
+		is >> util::binary_string(app_updates_url);
+		is >> util::binary_string(app_version);
 	} else {
 		app_support_url.clear(), app_updates_url.clear(), app_version.clear();
 	}
-	is >> util::encoded_string(default_dir_name, version.codepage());
-	is >> util::encoded_string(default_group_name, version.codepage());
+	is >> util::binary_string(default_dir_name);
+	is >> util::binary_string(default_group_name);
 	if(version < INNO_VERSION(3, 0, 0)) {
 		is >> util::ansi_string(uninstall_icon_name);
 	} else {
 		uninstall_icon_name.clear();
 	}
-	is >> util::encoded_string(base_filename, version.codepage());
-	if(version >= INNO_VERSION(1, 3, 21)) {
-		if(version < INNO_VERSION(5, 2, 5)) {
-			is >> util::ansi_string(license_text);
-			is >> util::ansi_string(info_before);
-			is >> util::ansi_string(info_after);
-		}
-		is >> util::encoded_string(uninstall_files_dir, version.codepage());
-		is >> util::encoded_string(uninstall_name, version.codepage());
-		is >> util::encoded_string(uninstall_icon, version.codepage());
-		is >> util::encoded_string(app_mutex, version.codepage());
+	is >> util::binary_string(base_filename);
+	if(version >= INNO_VERSION(1, 3, 0) && version < INNO_VERSION(5, 2, 5)) {
+		is >> util::ansi_string(license_text);
+		is >> util::ansi_string(info_before);
+		is >> util::ansi_string(info_after);
 	} else {
 		license_text.clear(), info_before.clear(), info_after.clear();
-		uninstall_files_dir.clear(), uninstall_name.clear();
-		uninstall_icon.clear(), app_mutex.clear();
+	}
+	if(version >= INNO_VERSION(1, 3, 3)) {
+		is >> util::binary_string(uninstall_files_dir);
+	} else {
+		uninstall_files_dir.clear();
+	}
+	if(version >= INNO_VERSION(1, 3, 6)) {
+		is >> util::binary_string(uninstall_name);
+		is >> util::binary_string(uninstall_icon);
+	} else {
+		uninstall_name.clear(), uninstall_icon.clear();
+	}
+	if(version >= INNO_VERSION(1, 3, 14)) {
+		is >> util::binary_string(app_mutex);
+	} else {
+		app_mutex.clear();
 	}
 	if(version >= INNO_VERSION(3, 0, 0)) {
-		is >> util::encoded_string(default_user_name, version.codepage());
-		is >> util::encoded_string(default_user_organisation, version.codepage());
+		is >> util::binary_string(default_user_name);
+		is >> util::binary_string(default_user_organisation);
 	} else {
 		default_user_name.clear(), default_user_organisation.clear();
 	}
-	if(version >= INNO_VERSION_EXT(3, 0, 6, 1)) {
-		is >> util::encoded_string(default_serial, version.codepage());
-		if(version < INNO_VERSION(5, 2, 5)) {
-			is >> util::binary_string(compiled_code);
-		}
+	if(version >= INNO_VERSION(4, 0, 0) || (version.is_isx() && version >= INNO_VERSION_EXT(3, 0, 6, 1))) {
+		is >> util::binary_string(default_serial);
 	} else {
-		default_serial.clear(), compiled_code.clear();
+		default_serial.clear();
+	}
+	if((version >= INNO_VERSION(4, 0, 0) && version < INNO_VERSION(5, 2, 5)) ||
+	   (version.is_isx() && version >= INNO_VERSION(1, 3, 24))) {
+		is >> util::binary_string(compiled_code);
+	} else {
+		compiled_code.clear();
 	}
 	if(version >= INNO_VERSION(4, 2, 4)) {
-		is >> util::encoded_string(app_readme_file, version.codepage());
-		is >> util::encoded_string(app_contact, version.codepage());
-		is >> util::encoded_string(app_comments, version.codepage());
-		is >> util::encoded_string(app_modify_path, version.codepage());
+		is >> util::binary_string(app_readme_file);
+		is >> util::binary_string(app_contact);
+		is >> util::binary_string(app_comments);
+		is >> util::binary_string(app_modify_path);
 	} else {
 		app_readme_file.clear(), app_contact.clear();
 		app_comments.clear(), app_modify_path.clear();
 	}
 	if(version >= INNO_VERSION(5, 3, 8)) {
-		is >> util::encoded_string(create_uninstall_registry_key, version.codepage());
+		is >> util::binary_string(create_uninstall_registry_key);
 	} else {
 		create_uninstall_registry_key.clear();
 	}
 	if(version >= INNO_VERSION(5, 3, 10)) {
-		is >> util::encoded_string(uninstallable, version.codepage());
+		is >> util::binary_string(uninstallable);
 	} else {
 		uninstallable.clear();
 	}
 	if(version >= INNO_VERSION(5, 5, 0)) {
-		is >> util::encoded_string(close_applications_filter, version.codepage());
+		is >> util::binary_string(close_applications_filter);
 	} else {
 		close_applications_filter.clear();
 	}
 	if(version >= INNO_VERSION(5, 5, 6)) {
-		is >> util::encoded_string(setup_mutex, version.codepage());
+		is >> util::binary_string(setup_mutex);
 	} else {
 		setup_mutex.clear();
 	}
+	if(version >= INNO_VERSION(5, 6, 1)) {
+		is >> util::binary_string(changes_environment);
+		is >> util::binary_string(changes_associations);
+	} else {
+		changes_environment.clear();
+		changes_associations.clear();
+	}
 	if(version >= INNO_VERSION(5, 2, 5)) {
 		is >> util::ansi_string(license_text);
 		is >> util::ansi_string(info_before);
@@ -250,7 +275,7 @@ void header::load(std::istream & is, const version & version) {
 		is >> util::binary_string(compiled_code);
 	}
 	
-	if(version >= INNO_VERSION(2, 0, 6) && !version.unicode) {
+	if(version >= INNO_VERSION(2, 0, 6) && !version.is_unicode()) {
 		lead_bytes = stored_char_set(is);
 	} else {
 		lead_bytes = 0;
@@ -276,38 +301,42 @@ void header::load(std::istream & is, const version & version) {
 		permission_count = 0;
 	}
 	
-	if(version >= INNO_VERSION(2, 0, 0)) {
+	if(version >= INNO_VERSION(2, 0, 0) || version.is_isx()) {
 		type_count = util::load<boost::uint32_t>(is);
 		component_count = util::load<boost::uint32_t>(is);
+	} else {
+		type_count = 0, component_count = 0;
+	}
+	if(version >= INNO_VERSION(2, 0, 0) || (version.is_isx() && version >= INNO_VERSION(1, 3, 17))) {
 		task_count = util::load<boost::uint32_t>(is);
 	} else {
-		type_count = 0, component_count = 0, task_count = 0;
+		task_count = 0;
 	}
 	
-	directory_count = util::load<boost::uint32_t>(is, version.bits);
-	file_count = util::load<boost::uint32_t>(is, version.bits);
-	data_entry_count = util::load<boost::uint32_t>(is, version.bits);
-	icon_count = util::load<boost::uint32_t>(is, version.bits);
-	ini_entry_count = util::load<boost::uint32_t>(is, version.bits);
-	registry_entry_count = util::load<boost::uint32_t>(is, version.bits);
-	delete_entry_count = util::load<boost::uint32_t>(is, version.bits);
-	uninstall_delete_entry_count = util::load<boost::uint32_t>(is, version.bits);
-	run_entry_count = util::load<boost::uint32_t>(is, version.bits);
-	uninstall_run_entry_count = util::load<boost::uint32_t>(is, version.bits);
+	directory_count = util::load<boost::uint32_t>(is, version.bits());
+	file_count = util::load<boost::uint32_t>(is, version.bits());
+	data_entry_count = util::load<boost::uint32_t>(is, version.bits());
+	icon_count = util::load<boost::uint32_t>(is, version.bits());
+	ini_entry_count = util::load<boost::uint32_t>(is, version.bits());
+	registry_entry_count = util::load<boost::uint32_t>(is, version.bits());
+	delete_entry_count = util::load<boost::uint32_t>(is, version.bits());
+	uninstall_delete_entry_count = util::load<boost::uint32_t>(is, version.bits());
+	run_entry_count = util::load<boost::uint32_t>(is, version.bits());
+	uninstall_run_entry_count = util::load<boost::uint32_t>(is, version.bits());
 	
 	boost::int32_t license_size = 0;
 	boost::int32_t info_before_size = 0;
 	boost::int32_t info_after_size = 0;
-	if(version < INNO_VERSION(1, 3, 21)) {
-		license_size = util::load<boost::int32_t>(is, version.bits);
-		info_before_size = util::load<boost::int32_t>(is, version.bits);
-		info_after_size = util::load<boost::int32_t>(is, version.bits);
+	if(version < INNO_VERSION(1, 3, 0)) {
+		license_size = util::load<boost::int32_t>(is, version.bits());
+		info_before_size = util::load<boost::int32_t>(is, version.bits());
+		info_after_size = util::load<boost::int32_t>(is, version.bits());
 	}
 	
 	winver.load(is, version);
 	
 	back_color = util::load<boost::uint32_t>(is);
-	if(version >= INNO_VERSION(1, 3, 21)) {
+	if(version >= INNO_VERSION(1, 3, 3)) {
 		back_color2 = util::load<boost::uint32_t>(is);
 	} else {
 		back_color2 = 0;
@@ -317,12 +346,22 @@ void header::load(std::istream & is, const version & version) {
 	} else {
 		image_back_color = 0;
 	}
-	if(version >= INNO_VERSION(2, 0, 0) && version < INNO_VERSION(5, 0, 4)) {
+	if((version >= INNO_VERSION(2, 0, 0) && version < INNO_VERSION(5, 0, 4)) || version.is_isx()) {
 		small_image_back_color = util::load<boost::uint32_t>(is);
 	} else {
 		small_image_back_color = 0;
 	}
 	
+	if(version >= INNO_VERSION(6, 0, 0)) {
+		wizard_style = stored_enum<stored_setup_style>(is).get();
+		wizard_resize_percent_x = util::load<boost::uint32_t>(is);
+		wizard_resize_percent_y = util::load<boost::uint32_t>(is);
+	} else {
+		wizard_style = ClassicStyle;
+		wizard_resize_percent_x = 0;
+		wizard_resize_percent_y = 0;
+	}
+	
 	if(version >= INNO_VERSION(5, 5, 7)) {
 		image_alpha_format = stored_enum<stored_alpha_format>(is).get();
 	} else {
@@ -355,30 +394,38 @@ void header::load(std::istream & is, const version & version) {
 		slices_per_disk = 1;
 	}
 	
-	if(version >= INNO_VERSION(2, 0, 0) && version < INNO_VERSION(5, 0, 0)) {
+	if((version >= INNO_VERSION(2, 0, 0) && version < INNO_VERSION(5, 0, 0)) ||
+	   (version.is_isx() && version >= INNO_VERSION(1, 3, 4))) {
 		install_mode = stored_enum<stored_install_verbosity>(is).get();
 	} else {
 		install_mode = NormalInstallMode;
 	}
 	
-	if(version >= INNO_VERSION(1, 3, 21)) {
+	if(version >= INNO_VERSION(1, 3, 0)) {
 		uninstall_log_mode = stored_enum<stored_log_mode>(is).get();
 	} else {
-		uninstall_log_mode = AppendLog;
+		uninstall_log_mode = NewLog;
 	}
 	
-	if(version >= INNO_VERSION(2, 0, 0) && version < INNO_VERSION(5, 0, 0)) {
+	if(version >= INNO_VERSION(5, 0, 0)) {
+		uninstall_style = ModernStyle;
+	} else if(version >= INNO_VERSION(2, 0, 0) || (version.is_isx() && version >= INNO_VERSION(1, 3, 13))) {
 		uninstall_style = stored_enum<stored_setup_style>(is).get();
 	} else {
-		uninstall_style = (version < INNO_VERSION(5, 0, 0)) ? ClassicStyle : ModernStyle;
+		uninstall_style = ClassicStyle;
 	}
 	
-	if(version >= INNO_VERSION(1, 3, 21)) {
+	if(version >= INNO_VERSION(1, 3, 6)) {
 		dir_exists_warning = stored_enum<stored_bool_auto_no_yes>(is).get();
 	} else {
 		dir_exists_warning = Auto;
 	}
 	
+	if(version.is_isx() && version >= INNO_VERSION(2, 0, 10) && version < INNO_VERSION(3, 0, 0)) {
+		boost::int32_t code_line_offset = util::load<boost::int32_t>(is);
+		(void)code_line_offset;
+	}
+	
 	if(version >= INNO_VERSION(3, 0, 0) && version < INNO_VERSION(3, 0, 3)) {
 		auto_bool val = stored_enum<stored_bool_auto_no_yes>(is).get();
 		switch(val) {
@@ -390,10 +437,16 @@ void header::load(std::istream & is, const version & version) {
 	
 	if(version >= INNO_VERSION(5, 3, 7)) {
 		privileges_required = stored_enum<stored_privileges_1>(is).get();
-	} else if(version >= INNO_VERSION(3, 0, 4)) {
+	} else if(version >= INNO_VERSION(3, 0, 4) || (version.is_isx() && version >= INNO_VERSION(3, 0, 3))) {
 		privileges_required = stored_enum<stored_privileges_0>(is).get();
 	}
 	
+	if(version >= INNO_VERSION(5, 7, 0)) {
+		privileges_required_override_allowed = stored_flags<stored_privileges_required_overrides>(is).get();
+	} else {
+		privileges_required_override_allowed = 0;
+	}
+	
 	if(version >= INNO_VERSION(4, 0, 10)) {
 		show_language_dialog = stored_enum<stored_bool_yes_no_auto>(is).get();
 		language_detection = stored_enum<stored_language_detection_method>(is).get();
@@ -453,7 +506,7 @@ void header::load(std::istream & is, const version & version) {
 		(void)util::load<boost::uint8_t>(is);
 	}
 	
-	stored_flag_reader<flags> flagreader(is, version.bits);
+	stored_flag_reader<flags> flagreader(is, version.bits());
 	
 	flagreader.add(DisableStartupPrompt);
 	if(version < INNO_VERSION(5, 3, 10)) {
@@ -463,7 +516,7 @@ void header::load(std::istream & is, const version & version) {
 	if(version < INNO_VERSION(5, 3, 3)) {
 		flagreader.add(DisableDirPage);
 	}
-	if(version < INNO_VERSION(1, 3, 21)) {
+	if(version < INNO_VERSION(1, 3, 6)) {
 		flagreader.add(DisableDirExistsWarning);
 	}
 	if(version < INNO_VERSION(5, 3, 3)) {
@@ -473,7 +526,7 @@ void header::load(std::istream & is, const version & version) {
 	if(version < INNO_VERSION(3, 0, 0) || version >= INNO_VERSION(3, 0, 3)) {
 		flagreader.add(AlwaysRestart);
 	}
-	if(version < INNO_VERSION(1, 3, 21)) {
+	if(version < INNO_VERSION(1, 3, 3)) {
 		flagreader.add(BackSolid);
 	}
 	flagreader.add(AlwaysUsePersonalGroup);
@@ -486,31 +539,45 @@ void header::load(std::istream & is, const version & version) {
 		flagreader.add(DisableAppendDir);
 	}
 	flagreader.add(Password);
-	flagreader.add(AllowRootDirectory);
-	flagreader.add(DisableFinishedPage);
-	if(version.bits != 16) {
+	if(version >= INNO_VERSION(1, 2, 6)) {
+		flagreader.add(AllowRootDirectory);
+	}
+	if(version >= INNO_VERSION(1, 2, 14)) {
+		flagreader.add(DisableFinishedPage);
+	}
+	if(version.bits() != 16) {
 		if(version < INNO_VERSION(3, 0, 4)) {
 			flagreader.add(AdminPrivilegesRequired);
 		}
 		if(version < INNO_VERSION(3, 0, 0)) {
 			flagreader.add(AlwaysCreateUninstallIcon);
 		}
-		if(version < INNO_VERSION(1, 3, 21)) {
+		if(version < INNO_VERSION(1, 3, 6)) {
 			flagreader.add(OverwriteUninstRegEntries);
 		}
-		flagreader.add(ChangesAssociations);
-	}
-	if(version >= INNO_VERSION(1, 3, 21)) {
-		if(version < INNO_VERSION(5, 3, 8)) {
-			flagreader.add(CreateUninstallRegKey);
+		if(version < INNO_VERSION(5, 6, 1)) {
+			flagreader.add(ChangesAssociations);
 		}
+	}
+	if(version >= INNO_VERSION(1, 3, 0) && version < INNO_VERSION(5, 3, 8)) {
+		flagreader.add(CreateUninstallRegKey);
+	}
+	if(version >= INNO_VERSION(1, 3, 1)) {
 		flagreader.add(UsePreviousAppDir);
+	}
+	if(version >= INNO_VERSION(1, 3, 3)) {
 		flagreader.add(BackColorHorizontal);
+	}
+	if(version >= INNO_VERSION(1, 3, 10)) {
 		flagreader.add(UsePreviousGroup);
+	}
+	if(version >= INNO_VERSION(1, 3, 20)) {
 		flagreader.add(UpdateUninstallLogAppName);
 	}
-	if(version >= INNO_VERSION(2, 0, 0)) {
+	if(version >= INNO_VERSION(2, 0, 0) || (version.is_isx() && version >= INNO_VERSION(1, 3, 10))) {
 		flagreader.add(UsePreviousSetupType);
+	}
+	if(version >= INNO_VERSION(2, 0, 0)) {
 		flagreader.add(DisableReadyMemo);
 		flagreader.add(AlwaysShowComponentsList);
 		flagreader.add(FlatComponentsList);
@@ -538,7 +605,7 @@ void header::load(std::istream & is, const version & version) {
 	if(version >= INNO_VERSION(3, 0, 3)) {
 		flagreader.add(RestartIfNeededByRun);
 	}
-	if(version >= INNO_VERSION_EXT(3, 0, 6, 1)) {
+	if(version >= INNO_VERSION(4, 0, 0) || (version.is_isx() && version >= INNO_VERSION(3, 0, 3))) {
 		flagreader.add(ShowTasksTreeLines);
 	}
 	if(version >= INNO_VERSION(4, 0, 0) && version < INNO_VERSION(4, 0, 10)) {
@@ -562,10 +629,10 @@ void header::load(std::istream & is, const version & version) {
 	if(version >= INNO_VERSION(4, 2, 2)) {
 		flagreader.add(EncryptionUsed);
 	}
-	if(version >= INNO_VERSION(5, 0, 4)) {
+	if(version >= INNO_VERSION(5, 0, 4) && version < INNO_VERSION(5, 6, 1)) {
 		flagreader.add(ChangesEnvironment);
 	}
-	if(version >= INNO_VERSION(5, 1, 7) && !version.unicode) {
+	if(version >= INNO_VERSION(5, 1, 7) && !version.is_unicode()) {
 		flagreader.add(ShowUndisplayableLanguages);
 	}
 	if(version >= INNO_VERSION(5, 1, 13)) {
@@ -590,6 +657,11 @@ void header::load(std::istream & is, const version & version) {
 	if(version >= INNO_VERSION(5, 5, 7)) {
 		flagreader.add(ForceCloseApplications);
 	}
+	if(version >= INNO_VERSION(6, 0, 0)) {
+		flagreader.add(AppNameHasConsts);
+		flagreader.add(UsePreviousPrivileges);
+		flagreader.add(WizardResizable);
+	}
 	
 	options |= flagreader;
 	
@@ -611,29 +683,61 @@ void header::load(std::istream & is, const version & version) {
 		disable_program_group_page = (options & DisableProgramGroupPage) ? Yes : No;
 	}
 	
-	if(version < INNO_VERSION(1, 3, 21)) {
+	if(version < INNO_VERSION(1, 3, 0)) {
 		if(license_size > 0) {
-			std::string temp;
-			temp.resize(size_t(license_size));
-			is.read(&temp[0], license_size);
-			util::to_utf8(temp, license_text);
+			license_text.resize(size_t(license_size));
+			is.read(&license_text[0], license_size);
+			util::to_utf8(license_text);
 		}
 		if(info_before_size > 0) {
-			std::string temp;
-			temp.resize(size_t(info_before_size));
-			is.read(&temp[0], info_before_size);
-			util::to_utf8(temp, info_before);
+			info_before.resize(size_t(info_before_size));
+			is.read(&info_before[0], info_before_size);
+			util::to_utf8(info_before);
 		}
 		if(info_after_size > 0) {
-			std::string temp;
-			temp.resize(size_t(info_after_size));
-			is.read(&temp[0], info_after_size);
-			util::to_utf8(temp, info_after);
+			info_after.resize(size_t(info_after_size));
+			is.read(&info_after[0], info_after_size);
+			util::to_utf8(info_after);
 		}
 	}
 	
 }
 
+void header::decode(util::codepage_id codepage) {
+	
+	util::to_utf8(app_name, codepage);
+	util::to_utf8(app_versioned_name, codepage);
+	util::to_utf8(app_id, codepage);
+	util::to_utf8(app_copyright, codepage);
+	util::to_utf8(app_publisher, codepage);
+	util::to_utf8(app_publisher_url, codepage);
+	util::to_utf8(app_support_phone, codepage);
+	util::to_utf8(app_support_url, codepage);
+	util::to_utf8(app_updates_url, codepage);
+	util::to_utf8(app_version, codepage);
+	util::to_utf8(default_dir_name, codepage);
+	util::to_utf8(default_group_name, codepage);
+	util::to_utf8(base_filename, codepage);
+	util::to_utf8(uninstall_files_dir, codepage);
+	util::to_utf8(uninstall_name, codepage);
+	util::to_utf8(uninstall_icon, codepage);
+	util::to_utf8(app_mutex, codepage);
+	util::to_utf8(default_user_name, codepage);
+	util::to_utf8(default_user_organisation, codepage);
+	util::to_utf8(default_serial, codepage);
+	util::to_utf8(app_readme_file, codepage);
+	util::to_utf8(app_contact, codepage);
+	util::to_utf8(app_comments, codepage);
+	util::to_utf8(app_modify_path, codepage);
+	util::to_utf8(create_uninstall_registry_key, codepage);
+	util::to_utf8(uninstallable, codepage);
+	util::to_utf8(close_applications_filter, codepage);
+	util::to_utf8(setup_mutex, codepage);
+	util::to_utf8(changes_environment, codepage);
+	util::to_utf8(changes_associations, codepage);
+	
+}
+
 } // namespace setup
 
 NAMES(setup::header::flags, "Setup Option",
@@ -708,6 +812,11 @@ NAMES(setup::header::architecture_types, "Architecture",
 	"ARM64",
 )
 
+NAMES(setup::header::privileges_required_overrides, "Priviledge Override"
+	"commandline",
+	"dialog",
+)
+
 NAMES(setup::header::alpha_format, "Alpha Format",
 	"ignored",
 	"defined",
@@ -726,7 +835,7 @@ NAMES(setup::header::log_mode, "Uninstall Log Mode",
 	"overwrite",
 )
 
-NAMES(setup::header::style, "Uninstall Style",
+NAMES(setup::header::style, "Style",
 	"classic",
 	"modern",
 )
