@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2014 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -31,9 +31,11 @@
 
 #include <boost/cstdint.hpp>
 
+#include "util/encoding.hpp"
+
 namespace setup {
 
-struct version;
+struct info;
 
 struct language_entry {
 	
@@ -60,7 +62,9 @@ struct language_entry {
 	
 	bool right_to_left;
 	
-	void load(std::istream & is, const version & version);
+	void load(std::istream & is, const info & i);
+	
+	void decode(util::codepage_id cp);
 	
 };
 
