@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2014 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -35,7 +35,7 @@
 
 namespace setup {
 
-struct version;
+struct info;
 
 struct run_entry : public item {
 	
@@ -45,7 +45,7 @@ struct run_entry : public item {
 		PostInstall,
 		Unchecked,
 		SkipIfSilent,
-		Skipif_not_equalSilent,
+		SkipIfNotSilent,
 		HideWizard,
 		Bits32,
 		Bits64,
@@ -72,7 +72,7 @@ struct run_entry : public item {
 	
 	flags options;
 	
-	void load(std::istream & is, const version & version);
+	void load(std::istream & is, const info & i);
 	
 };
 
