@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -18,8 +18,6 @@
  * 3. This notice may not be removed or altered from any source distribution.
  */
 
-#define INNOEXTRACT_HAVE_LZMA 1
-
 #include "stream/lzma.hpp"
 
 #include <boost/cstdint.hpp>
@@ -108,7 +106,7 @@ bool inno_lzma1_decompressor_impl::filter(const char * & begin_in, const char *
 		}
 		options.pb = boost::uint32_t(properties / (9 * 5));
 		options.lp = boost::uint32_t((properties % (9 * 5)) / 9);
-		options.lc = properties % 9;
+		options.lc = boost::uint32_t(properties % 9);
 		
 		options.dict_size = util::little_endian::load<boost::uint32_t>(header + 1);
 		
