@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Daniel Scharrer
+ * Copyright (C) 2011-2019 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -78,99 +78,197 @@
 
 namespace util {
 
-enum known_codepages {
-	cp_utf16le = 1200,
-	cp_windows1252 = 1252,
-	cp_ascii = 20127,
-	cp_iso_8859_1 = 28591,
-	cp_utf8  = 65001,
-};
-
 namespace {
 
 //! Get names for encodings where iconv doesn't have the codepage alias
 const char * get_encoding_name(codepage_id codepage) {
 	switch(codepage) {
-		case   708: return "ISO-8859-6";
-		case   936: return "GBK";
-		case   949: return "UHC";
-		case   950: return "BIG5";
-		// iconv's behavior for "UTF-16" is platform-dependent if there is no BOM.
-		// There never is any BOM in Inno Setup files and it's always little-endian,
-		// so we specify the exact encoding.
-		case  1200: return "UTF-16LE";
-		case  1201: return "UTF-16BE";
-		case  1252: return "MS-ANSI";
-		case  1361: return "JOHAB";
-		case 10000: return "MACINTOSH";
-		case 10002: return "BIG5";
-		case 10008: return "GB2312";
-		case 12000: return "UTF-32LE";
-		case 12001: return "UTF-32BE";
-		case 20003: return "IBM5550";
-		case 20127: return "US-ASCII";
-		case 20261: return "T.61";
-		case 20269: return "ISO_6937";
-		case 20273: return "IBM273";
-		case 20277: return "IBM277";
-		case 20278: return "IBM278";
-		case 20280: return "IBM280";
-		case 20284: return "IBM284";
-		case 20285: return "IBM285";
-		case 20290: return "IBM290";
-		case 20297: return "IBM297";
-		case 20420: return "IBM420";
-		case 20423: return "IBM423";
-		case 20424: return "IBM424";
-		case 20866: return "KOI8-R";
-		case 20871: return "IBM871";
-		case 20880: return "IBM880";
-		case 20905: return "IBM905";
-		case 20924: return "IBM1047";
-		case 20932: return "EUC-JP-MS";
-		case 20936: return "EUC-CN";
-		case 21025: return "IBM1025";
-		case 21866: return "KOI8-U";
-		case 28591: return "ISO-8859-1";
-		case 28592: return "ISO-8859-2";
-		case 28593: return "ISO-8859-3";
-		case 28594: return "ISO-8859-4";
-		case 28595: return "ISO-8859-5";
-		case 28596: return "ISO-8859-6";
-		case 28597: return "ISO-8859-7";
-		case 28598: return "ISO-8859-8";
-		case 28599: return "ISO-8859-9";
-		case 28603: return "ISO-8859-13";
-		case 28605: return "ISO-8859-15";
-		case 38598: return "ISO-8859-8";
-		case 50220: return "ISO-2022-JP";
-		case 50221: return "ISO-2022-JP-2";
-		case 50222: return "ISO-2022-JP-3";
-		case 50225: return "ISO-2022-KR";
-		case 50227: return "ISO-2022-CN";
-		case 50229: return "ISO-2022-CN-EXT";
-		case 50930: return "EBCDIC-JP-E";
-		case 51932: return "EUC-JP";
-		case 51936: return "EUC-CN";
-		case 51949: return "EUC-KR";
-		case 51950: return "EUC-CN";
-		case 54936: return "GB18030";
-		case 65000: return "UTF-7";
-		case 65001: return "UTF-8";
+		case cp_ascii:        return "US-ASCII";
+		case cp_big5:         return "BIG5";
+		case cp_big5_eten:    return "BIG5";
+		case cp_big5_hkscs:   return "BIG5-HKSCS";
+		case cp_cns:          return "EUC-TW";
+		case cp_dos708:       return "ISO-8859-6";
+		case cp_euc_cn:       return "EUC-CN";
+		case cp_euc_jp:       return "EUC-JP";
+		case cp_euc_jp_ms:    return "EUC-JP-MS";
+		case cp_euc_kr:       return "EUC-KR";
+		case cp_euc_tw:       return "EUC-TW";
+		case cp_gb2312_80:    return "GB2312";
+		case cp_gb2312_hz:    return "GB2312";
+		case cp_gb18030:      return "GB18030";
+		case cp_gbk:          return "GBK";
+		case cp_ia5:          return "ISO_646.IRV:1991";
+		case cp_ia5_de:       return "ISO646-DE";
+		case cp_ia5_no2:      return "ISO646-NO2";
+		case cp_ia5_se2:      return "ISO646-SE2";
+		case cp_ibm273:       return "IBM273";
+		case cp_ibm277:       return "IBM277";
+		case cp_ibm278:       return "IBM278";
+		case cp_ibm280:       return "IBM280";
+		case cp_ibm284:       return "IBM284";
+		case cp_ibm285:       return "IBM285";
+		case cp_ibm290:       return "IBM290";
+		case cp_ibm297:       return "IBM297";
+		case cp_ibm420:       return "IBM420";
+		case cp_ibm423:       return "IBM423";
+		case cp_ibm424:       return "IBM424";
+		case cp_ibm833:       return "IBM833";
+		case cp_ibm838:       return "IBM1160";
+		case cp_ibm871:       return "IBM871";
+		case cp_ibm880:       return "IBM880";
+		case cp_ibm905:       return "IBM905";
+		case cp_ibm924:       return "IBM1047";
+		case cp_ibm930:       return "IBM930";
+		case cp_ibm931:       return "IBM931";
+		case cp_ibm933:       return "IBM933";
+		case cp_ibm935:       return "IBM935";
+		case cp_ibm936:       return "IBM936";
+		case cp_ibm937:       return "IBM937";
+		case cp_ibm939:       return "IBM939";
+		case cp_ibm1025:      return "IBM1025";
+		case cp_iso_2022_cn:  return "ISO-2022-CN";
+		case cp_iso_2022_cn2: return "ISO-2022-CN-EXT";
+		case cp_iso_2022_jp:  return "ISO-2022-JP";
+		case cp_iso_2022_jp2: return "ISO-2022-JP-2";
+		case cp_iso_2022_jp3: return "ISO-2022-JP-3";
+		case cp_iso_2022_kr:  return "ISO-2022-KR";
+		case cp_iso_6937:     return "ISO_6937";
+		case cp_iso_8859_10:  return "ISO-8859-10";
+		case cp_iso_8859_11:  return "ISO-8859-11";
+		case cp_iso_8859_13:  return "ISO-8859-13";
+		case cp_iso_8859_14:  return "ISO-8859-14";
+		case cp_iso_8859_15:  return "ISO-8859-15";
+		case cp_iso_8859_1:   return "ISO-8859-1";
+		case cp_iso_8859_2:   return "ISO-8859-2";
+		case cp_iso_8859_3:   return "ISO-8859-3";
+		case cp_iso_8859_4:   return "ISO-8859-4";
+		case cp_iso_8859_5:   return "ISO-8859-5";
+		case cp_iso_8859_6:   return "ISO-8859-6";
+		case cp_iso_8859_6i:  return "ISO-8859-6";
+		case cp_iso_8859_7:   return "ISO-8859-7";
+		case cp_iso_8859_8:   return "ISO-8859-8";
+		case cp_iso_8859_8i:  return "ISO-8859-8";
+		case cp_iso_8859_9:   return "ISO-8859-9";
+		case cp_johab:        return "JOHAB";
+		case cp_koi8_r:       return "KOI8-R";
+		case cp_koi8_u:       return "KOI8-U";
+		case cp_macarabic:    return "MACARABIC";
+		case cp_macchinese1:  return "BIG5";
+		case cp_macchinese2:  return "EUC-CN";
+		case cp_maccroatian:  return "MACCROATIAN";
+		case cp_maccyrillic:  return "MACCYRILLIC";
+		case cp_macgreek:     return "MACGREEK";
+		case cp_machebrew:    return "MACHEBREW";
+		case cp_maciceland:   return "MACICELAND";
+		case cp_macjapanese:  return "SHIFT-JIS";
+		case cp_mackorean:    return "EUC-KR";
+		case cp_macroman2:    return "MACCENTRALEUROPE";
+		case cp_macroman:     return "MACINTOSH";
+		case cp_macromania:   return "MACROMANIA";
+		case cp_macthai:      return "MACTHAI";
+		case cp_macturkish:   return "MACTURKISH";
+		case cp_macukraine:   return "MACUKRAINE";
+		case cp_shift_jis:    return "SHIFT-JIS";
+		case cp_t61:          return "T.61";
+		case cp_uhc:          return "UHC";
+		case cp_utf7:         return "UTF-7";
+		case cp_utf8:         return "UTF-8";
+		case cp_utf16be:      return "UTF-16BE";
+		case cp_utf16le:      return "UTF-16LE"; // "UTF-16" is platform-dependent without a BOM
+		case cp_utf32be:      return "UTF-32BE";
+		case cp_utf32le:      return "UTF-32LE";
+		case cp_wansung:      return "EUC-KR";
+		case cp_windows1250:  return "MS-EE";
+		case cp_windows1251:  return "MS-CYRL";
+		case cp_windows1252:  return "MS-ANSI";
+		case cp_windows1253:  return "MS-GREEK";
+		case cp_windows1254:  return "MS-TURK";
+		case cp_windows1255:  return "MS-HEBR";
+		case cp_windows1256:  return "MS-ARAB";
 		default: return NULL;
 	}
 }
 
+//! Check if a codepage is known to be a superset of ASCII - used for optimization only
+bool is_extended_ascii(codepage_id codepage) {
+	
+	// cp_utf8 is handled separately
+	
+	if(codepage >= cp_windows1250 && codepage <= cp_windows1270) {
+		return true;
+	}
+	
+	if(codepage >= cp_iso_8859_1 && codepage <= cp_iso_8859_15) {
+		return true;
+	}
+	
+	switch(codepage) {
+		case cp_ascii:
+		case cp_big5:
+		case cp_big5_eten:
+		case cp_big5_hkscs:
+		case cp_cns:
+		case cp_dos708:
+		case cp_euc_cn:
+		case cp_euc_tw:
+		case cp_gb18030:
+		case cp_gbk:
+		case cp_iso_6937:
+		case cp_iso_8859_6i:
+		case cp_iso_8859_8i:
+		case cp_koi8_r:
+		case cp_koi8_u:
+		case cp_macarabic:
+		case cp_macchinese1:
+		case cp_macchinese2:
+		case cp_maccyrillic:
+		case cp_macgreek:
+		case cp_maciceland:
+		case cp_macroman:
+		case cp_uhc:
+		case cp_windows874:
+			return true;
+		default:
+			return false;
+	}
+	
+}
+
+bool is_ascii(const std::string & data) {
+	// String in an extended ASCII encoding contains only ASCII characters
+	BOOST_FOREACH(char c, data) {
+		if(boost::uint8_t(c) >= 128) {
+			return false;
+		}
+	}
+	return true;
+}
+
+//! Check if a string is compatible with UTF-8
+bool is_utf8(const std::string & data, codepage_id codepage) {
+	
+	if(codepage == cp_utf8 || codepage == cp_ascii) {
+		return true;
+	}
+	
+	if(is_extended_ascii(codepage) && is_ascii(data)) {
+		return true;
+	}
+	
+	return false;
+}
+
 typedef boost::uint32_t unicode_char;
 
 const unicode_char replacement_char = '_';
 
-size_t get_encoding_size(codepage_id codepage) {
+size_t get_code_unit_size(codepage_id codepage) {
 	switch(codepage) {
-		case  1200: return 2u; // UTF-16LE
-		case  1201: return 2u; // UTF-16BE
-		case 12000: return 4u; // UTF-32LE
-		case 12001: return 4u; // UTF-32BE
+		case cp_utf16le: return 2u;
+		case cp_utf16be: return 2u;
+		case cp_utf32le: return 4u;
+		case cp_utf32be: return 4u;
 		default:    return 1u;
 	}
 }
@@ -178,12 +276,12 @@ size_t get_encoding_size(codepage_id codepage) {
 //! Fallback conversion that will at least work for ASCII characters
 void to_utf8_fallback(const std::string & from, std::string & to, codepage_id codepage) {
 	
-	size_t skip = get_encoding_size(codepage);
+	size_t skip = get_code_unit_size(codepage);
 	
 	size_t shift = 0;
 	switch(codepage) {
-		case  1201: shift = 1u * 8u; break; // UTF-16BE
-		case 12001: shift = 3u * 8u; break; // UTF-32BE
+		case cp_utf16be: shift = 1u * 8u; break;
+		case cp_utf32be: shift = 3u * 8u; break;
 		default: break;
 	}
 	
@@ -324,7 +422,9 @@ bool is_utf16_low_surrogate(unicode_char chr) {
 	return chr >= 0xdc00 && chr <= 0xdfff;
 }
 
-void utf16le_to_utf8(const std::string & from, std::string & to) {
+} // anonymous namespace
+
+void utf16le_to_wtf8(const std::string & from, std::string & to) {
 	
 	if(from.size() % 2 != 0) {
 		log_warning << "Unexpected trailing byte in UTF-16 string.";
@@ -337,49 +437,30 @@ void utf16le_to_utf8(const std::string & from, std::string & to) {
 	
 	std::string::const_iterator it = from.begin();
 	std::string::const_iterator end = from.end();
+	if(from.size() % 2 != 0) {
+		--end;
+	}
 	while(it != end) {
 		
 		unicode_char chr = boost::uint8_t(*it++);
-		if(it == end) {
-			warn = true;
-			utf8_write(to, replacement_char);
-			break;
-		}
 		chr |= unicode_char(boost::uint8_t(*it++)) << 8;
 		
 		// If it's a surrogate pair, convert to a single UTF-32 character
-		if(is_utf16_high_surrogate(chr)) {
-			if(it == end) {
-				warn = true;
-				utf8_write(to, replacement_char);
-				break;
-			}
-			unicode_char d = boost::uint8_t(*it++);
-			if(it == end) {
-				warn = true;
-				utf8_write(to, replacement_char);
-				break;
-			}
-			d |= unicode_char(boost::uint8_t(*it++)) << 8;
+		if(is_utf16_high_surrogate(chr) && it != end) {
+			unicode_char d = boost::uint8_t(*it);
+			d |= unicode_char(boost::uint8_t(*(it + 1))) << 8;
 			if(is_utf16_low_surrogate(d)) {
 				chr = ((chr - 0xd800) << 10) + (d - 0xdc00) + 0x0010000;
-			} else {
-				warn = true;
-				utf8_write(to, replacement_char);
-				continue;
+				it += 2;
 			}
 		}
 		
-		// Replace invalid characters
-		if(chr > 0x0010FFFF) {
-			warn = true;
-			// Invalid character (greater than the maximum unicode value)
-			utf8_write(to, replacement_char);
-			continue;
-		}
-		
 		utf8_write(to, chr);
 	}
+	if(end != from.end()) {
+		warn = true;
+		utf8_write(to, replacement_char);
+	}
 	
 	if(warn) {
 		log_warning << "Unexpected data while converting from UTF-16LE to UTF-8.";
@@ -387,21 +468,16 @@ void utf16le_to_utf8(const std::string & from, std::string & to) {
 	
 }
 
-void utf8_to_utf16le(const std::string & from, std::string & to) {
+void wtf8_to_utf16le(const std::string & from, std::string & to) {
 	
 	to.clear();
 	to.reserve(from.size() * 2); // optimistically, most strings only have ASCII characters
 	
-	bool warn = false;
-	
 	for(std::string::const_iterator i = from.begin(); i != from.end(); ) {
 		
 		unicode_char chr = utf8_read(i, from.end());
 		
-		if((chr >= 0xd800 && chr <= 0xdfff) || chr > 0x10ffff) {
-			chr = replacement_char;
-			warn = true;
-		} else if(chr >= 0x10000) {
+		if(chr >= 0x10000) {
 			chr -= 0x10000;
 			unicode_char high_surrogate = 0xd800 + (chr >> 10);
 			to.push_back(char(boost::uint8_t(high_surrogate)));
@@ -413,12 +489,10 @@ void utf8_to_utf16le(const std::string & from, std::string & to) {
 		to.push_back(char(boost::uint8_t(chr >> 8)));
 	}
 	
-	if(warn) {
-		log_warning << "Unexpected data while converting from UTF-8 to UTF-16LE.";
-	}
-	
 }
 
+namespace {
+
 unicode_char windows1252_replacements[] = {
 	0x20ac, replacement_char, 0x201a, 0x192, 0x201e, 0x2026, 0x2020, 0x2021, 0x2c6,
 	0x2030, 0x160, 0x2039, 0x152, replacement_char, 0x17d, replacement_char,
@@ -466,14 +540,14 @@ void utf8_to_windows1252(const std::string & from, std::string & to) {
 		
 		// Windows-1252 maps almost directly to Unicode - yay!
 		if(chr >= 256 || (chr >= 128 && chr < 160)) {
-			size_t i = 0;
-			for(; i < size_t(boost::size(windows1252_replacements)); i++) {
-				if(chr == windows1252_replacements[i] && windows1252_replacements[i] != replacement_char) {
+			size_t j = 0;
+			for(; j < size_t(boost::size(windows1252_replacements)); j++) {
+				if(chr == windows1252_replacements[j] && windows1252_replacements[j] != replacement_char) {
 					break;
 				}
 			}
-			if(i < size_t(boost::size(windows1252_replacements))) {
-				chr = unicode_char(128 + i);
+			if(j < size_t(boost::size(windows1252_replacements))) {
+				chr = unicode_char(128 + j);
 			} else {
 				chr = replacement_char;
 				warn = true;
@@ -494,10 +568,12 @@ void utf8_to_windows1252(const std::string & from, std::string & to) {
 typedef boost::unordered_map<codepage_id, iconv_t> converter_map;
 converter_map converters;
 
-iconv_t get_converter(codepage_id codepage) {
+iconv_t get_converter(codepage_id codepage, bool reverse) {
+	
+	boost::uint32_t key = codepage | (reverse ? 0x80000000 : 0);
 	
 	// Try to reuse an existing converter if possible
-	converter_map::const_iterator i = converters.find(codepage);
+	converter_map::const_iterator i = converters.find(key);
 	if(i != converters.end()) {
 		return i->second;
 	}
@@ -506,7 +582,7 @@ iconv_t get_converter(codepage_id codepage) {
 	
 	const char * encoding = get_encoding_name(codepage);
 	if(encoding) {
-		handle = iconv_open("UTF-8", encoding);
+		handle = reverse ? iconv_open(encoding, "UTF-8") : iconv_open("UTF-8", encoding);
 	}
 	
 	// Otherwise, try a few different codepage name prefixes
@@ -515,7 +591,7 @@ iconv_t get_converter(codepage_id codepage) {
 		BOOST_FOREACH(const char * prefix, prefixes) {
 			std::ostringstream oss;
 			oss << prefix << std::setfill('0') << std::setw(3) << codepage;
-			handle = iconv_open("UTF-8", oss.str().c_str());
+			handle = reverse ? iconv_open(oss.str().c_str(), "UTF-8") : iconv_open("UTF-8", oss.str().c_str());
 			if(handle != iconv_t(-1)) {
 				break;
 			}
@@ -526,12 +602,12 @@ iconv_t get_converter(codepage_id codepage) {
 		log_warning << "Could not get codepage " << codepage << " -> UTF-8 converter.";
 	}
 	
-	return converters[codepage] = handle;
+	return converters[key] = handle;
 }
 
-bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id codepage) {
+bool utf8_iconv(const std::string & from, std::string & to, codepage_id codepage, bool reverse) {
 	
-	iconv_t converter = get_converter(codepage);
+	iconv_t converter = get_converter(codepage, reverse);
 	if(converter == iconv_t(-1)) {
 		return false;
 	}
@@ -554,7 +630,7 @@ bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id codep
 	
 	iconv(converter, NULL, NULL, NULL, NULL);
 	
-	size_t skip = get_encoding_size(codepage);
+	size_t skip = get_code_unit_size(codepage);
 	
 	bool warn = false;
 	
@@ -591,7 +667,11 @@ bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id codep
 	}
 	
 	if(warn) {
-		log_warning << "Unexpected data while converting from CP" << codepage << " to UTF-8.";
+		if(reverse) {
+			log_warning << "Unexpected data while converting from UTF-8 to CP" << codepage << ".";
+		} else {
+			log_warning << "Unexpected data while converting from CP" << codepage << " to UTF-8.";
+		}
 	}
 	
 	to.resize(outbase);
@@ -599,6 +679,14 @@ bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id codep
 	return true;
 }
 
+bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id codepage) {
+	return utf8_iconv(from, to, codepage, false);
+}
+
+bool from_utf8_iconv(const std::string & from, std::string & to, codepage_id codepage) {
+	return utf8_iconv(from, to, codepage, true);
+}
+
 #endif // INNOEXTRACT_HAVE_ICONV
 
 #if INNOEXTRACT_HAVE_WIN32_CONV
@@ -606,8 +694,7 @@ bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id codep
 std::string windows_error_string(DWORD code) {
 	char * error;
 	DWORD n = FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
-	                         NULL, code, 0, reinterpret_cast<char *>(&error), 0,
-	                         NULL);
+	                         NULL, code, 0, reinterpret_cast<char *>(&error), 0, NULL);
 	if(n == 0) {
 		return "unknown";
 	} else {
@@ -621,40 +708,41 @@ std::string windows_error_string(DWORD code) {
 }
 
 bool to_utf8_win32(const std::string & from, std::string & to, codepage_id codepage) {
-	
-	int ret = 0;
-	
+	 
 	// Convert from the source codepage to UTF-16LE
-	const WCHAR * utf16;
-	int utf16_size;
-	std::vector<WCHAR> buffer;
-	if(codepage == cp_utf16le) {
-		utf16 = reinterpret_cast<const WCHAR *>(from.data());
-		utf16_size = int(from.size()) / 2;
-	} else {
-		utf16_size = MultiByteToWideChar(codepage, 0, from.data(), int(from.length()), NULL, 0);
-		if(utf16_size > 0) {
-			buffer.resize(size_t(utf16_size));
-			ret = MultiByteToWideChar(codepage, 0, from.data(), int(from.length()),
-			                          &buffer.front(), utf16_size);
-		}
-		if(utf16_size <= 0 || ret <= 0) {
-			log_warning << "Error while converting from CP" << codepage << " to UTF-16: "
-			            << windows_error_string(GetLastError());
-			return false;
-		}
-		utf16 = &buffer.front();
+	std::string buffer;
+	int ret = MultiByteToWideChar(codepage, 0, from.data(), int(from.length()), NULL, 0);
+	if(ret > 0) {
+		buffer.resize(size_t(ret) * 2);
+		ret = MultiByteToWideChar(codepage, 0, from.data(), int(from.length()),
+		                          reinterpret_cast<LPWSTR>(&buffer[0]), ret);
+	}
+	if(ret <= 0) {
+		log_warning << "Error while converting from CP" << codepage << " to UTF-16: "
+		            << windows_error_string(GetLastError());
+		return false;
 	}
 	
-	// Convert from UTF-16LE to UTF-8
-	int utf8_size = WideCharToMultiByte(CP_UTF8, 0, utf16, utf16_size, NULL, 0,  NULL, NULL);
-	if(utf8_size > 0) {
-		to.resize(size_t(utf8_size));
-		ret = WideCharToMultiByte(CP_UTF8, 0, utf16, utf16_size,
-		                          &to[0], utf8_size, NULL, NULL);
+	utf16le_to_wtf8(buffer, to);
+	
+	return true;
+}
+
+bool from_utf8_win32(const std::string & from, std::string & to, codepage_id codepage) {
+	
+	std::string buffer;
+	wtf8_to_utf16le(from, buffer);
+	
+	// Convert from UTF-16LE to the target codepage
+	LPCWSTR data = reinterpret_cast<LPCWSTR>(buffer.c_str());
+	int size = int(buffer.size() / 2);
+	int ret = WideCharToMultiByte(codepage, 0, data, size, NULL, 0,  NULL, NULL);
+	if(ret > 0) {
+		to.resize(size_t(ret));
+		ret = WideCharToMultiByte(codepage, 0, data, size, &to[0], ret, NULL, NULL);
 	}
-	if(utf8_size <= 0 || ret <= 0) {
-		log_warning << "Error while converting from UTF-16 to UTF-8: "
+	if(ret <= 0) {
+		log_warning << "Error while converting from UTF-16 to CP" << codepage << ": "
 		            << windows_error_string(GetLastError());
 		return false;
 	}
@@ -664,22 +752,10 @@ bool to_utf8_win32(const std::string & from, std::string & to, codepage_id codep
 
 #endif // INNOEXTRACT_HAVE_WIN32_CONV
 
-} // anonymous namespace
-
 void to_utf8(const std::string & from, std::string & to, codepage_id codepage) {
 	
-	if(from.empty()) {
-		to.clear();
-		return;
-	}
-	
-	if(codepage == cp_utf8 || codepage == cp_ascii) {
-		to = from;
-		return;
-	}
-	
 	switch(codepage) {
-		case cp_utf16le:     utf16le_to_utf8(from, to); return;
+		case cp_utf16le:     utf16le_to_wtf8(from, to); return;
 		case cp_windows1252: windows1252_to_utf8(from, to); return;
 		case cp_iso_8859_1:  windows1252_to_utf8(from, to); return;
 		default: break;
@@ -701,6 +777,20 @@ void to_utf8(const std::string & from, std::string & to, codepage_id codepage) {
 	
 }
 
+} // anonymous namespace
+
+void to_utf8(std::string & data, codepage_id codepage) {
+	
+	if(data.empty() || is_utf8(data, codepage)) {
+		// Already UTF-8
+		return;
+	}
+	
+	std::string buffer;
+	to_utf8(data, buffer, codepage);
+	std::swap(data, buffer);
+}
+
 void from_utf8(const std::string & from, std::string & to, codepage_id codepage) {
 	
 	if(from.empty()) {
@@ -708,21 +798,32 @@ void from_utf8(const std::string & from, std::string & to, codepage_id codepage)
 		return;
 	}
 	
-	if(codepage == cp_utf8) {
+	if(codepage == cp_utf8 || (is_extended_ascii(codepage) && is_ascii(from))) {
 		to = from;
 		return;
 	}
 	
 	switch(codepage) {
-		case cp_utf16le:     utf8_to_utf16le(from, to); return;
+		case cp_utf16le:     wtf8_to_utf16le(from, to); return;
 		case cp_windows1252: utf8_to_windows1252(from, to); return;
-		case cp_iso_8859_1:  utf8_to_windows1252(from, to); return;
-		default: {
-			log_warning << "Unsupported output codepage: " << codepage;
-			to = from;
-		}
+		default: break;
 	}
 	
+	#if INNOEXTRACT_HAVE_ICONV
+	if(from_utf8_iconv(from, to, codepage)) {
+		return;
+	}
+	#endif
+	
+	#if INNOEXTRACT_HAVE_WIN32_CONV
+	if(from_utf8_win32(from, to, codepage)) {
+		return;
+	}
+	#endif
+	
+	log_warning << "Unsupported output codepage: " << codepage;
+	to = from;
+	
 }
 
 std::string encoding_name(codepage_id codepage) {
